package fr.service.interceptor;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.XMLConstants;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.message.Message;
import org.apache.cxf.phase.AbstractPhaseInterceptor;
import org.apache.cxf.phase.Phase;
import org.apache.cxf.service.Service;
import org.apache.cxf.service.model.ServiceInfo;
import org.apache.cxf.service.model.ServiceModelUtil;
import org.springframework.core.io.Resource;
import org.xml.sax.SAXException;

public class SetShemaInterceptor extends
		AbstractPhaseInterceptor<Message> {

	private Resource[] resources;
	private String targetNamespace;
	private Schema schema = null;

	public SetShemaInterceptor() {
		super(Phase.PRE_PROTOCOL);
	}

	public void handleMessage(Message message) throws Fault {
		Service service = ServiceModelUtil.getService(message.getExchange());
		ServiceInfo serviceInfo = service.getServiceInfos().get(0);
		if (schema == null) {
			schema = initSchema(message);
		}
		serviceInfo.setProperty(Schema.class.getName(), schema);
	}

	/**
	 * 
	 * Multiple schema import...
	 * http://stackoverflow.com/questions/1094893/validate-an-xml-file-against-multiple-schema-definitions
	 * 
	 * @param message
	 * @return
	 */
	private synchronized Schema initSchema(Message message) {

		try {	
			SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
			String xmlXsdSchema =
			"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"
			   + "<xs:schema xmlns:xs=\""+XMLConstants.W3C_XML_SCHEMA_NS_URI+"\""
			   + " targetNamespace=\""+getTargetNamespace()+"\"" 
			   + " elementFormDefault=\"qualified\">\n";
			for (int i = 0; i < resources.length; i++) {
				xmlXsdSchema +="<xs:include schemaLocation=\"" +resources[i].getURL() +"\"/>\n";
			}
			xmlXsdSchema +="</xs:schema>";
			Schema schema = schemaFactory.newSchema(new StreamSource(new StringReader(xmlXsdSchema), "xsdTop"));

			return schema;
		} catch (SAXException e) {
			throw new Fault(e);
		} catch (IOException e) {
			throw new Fault(e);
		}
	}
	
	/**
	 * @return the resources
	 */
	public Resource[] getResources() {
		return resources;
	}

	/**
	 * @param resources
	 *            the resources to set
	 */
	public void setResources(Resource[] resources) {
		this.resources = resources;
	}

	/**
	 * @return the targetNamespace
	 */
	public String getTargetNamespace() {
		return targetNamespace;
	}

	/**
	 * @param targetNamespace the targetNamespace to set
	 */
	public void setTargetNamespace(String targetNamespace) {
		this.targetNamespace = targetNamespace;
	}

}
